@@ -63,11 +63,7 @@ void KoCanvasControllerWidget::Private::setDocumentOffset()
         // If it isn't an OpenGL canvas
         if (qobject_cast<QOpenGLWidget*>(canvasWidget) == 0) {
             QPoint diff = q->documentOffset() - pt;
-            if (q->canvasMode() == Spreadsheet && canvasWidget->layoutDirection() == Qt::RightToLeft) {
-                canvasWidget->scroll(-diff.x(), diff.y());
-            } else {
-                canvasWidget->scroll(diff.x(), diff.y());
-            }
+            canvasWidget->scroll(diff.x(), diff.y(), canvasWidget->rect());
         }
     }
 
@@ -231,11 +227,6 @@ QSize KoCanvasControllerWidget::viewportSize() const
     return viewport()->size();
 }
 
-void KoCanvasControllerWidget::setDrawShadow(bool drawShadow)
-{
-    d->viewportWidget->setDrawShadow(drawShadow);
-}
-
 void KoCanvasControllerWidget::resizeEvent(QResizeEvent *resizeEvent)
 {
     proxyObject->emitSizeChanged(resizeEvent->size());
