@@ -146,11 +146,24 @@ const KisTimeRange& KisImageAnimationInterface::fullClipRange() const
     return m_d->fullClipRange;
 }
 
-void KisImageAnimationInterface::setFullClipRange(const KisTimeRange range) {
+void KisImageAnimationInterface::setFullClipRange(const KisTimeRange range)
+{
     m_d->fullClipRange = range;
     emit sigFullClipRangeChanged();
 }
 
+void KisImageAnimationInterface::setFullClipRangeStartTime(int column)
+{
+    KisTimeRange newRange(column,  m_d->fullClipRange.end(), false);
+    setFullClipRange(newRange);
+}
+
+void KisImageAnimationInterface::setFullClipRangeEndTime(int column)
+{
+    KisTimeRange newRange(m_d->fullClipRange.start(), column, false);
+    setFullClipRange(newRange);
+}
+
 const KisTimeRange& KisImageAnimationInterface::playbackRange() const
 {
     return m_d->playbackRange.isValid() ? m_d->playbackRange : m_d->fullClipRange;
@@ -345,21 +358,16 @@ void KisImageAnimationInterface::notifyNodeChanged(const KisNode *node,
     if (externalFrameActive() || m_d->frameInvalidationBlocked) return;
     if (node->inherits("KisSelectionMask")) return;
 
-    KisKeyframeChannel *channel =
-        node->getKeyframeChannel(KisKeyframeChannel::Content.id());
 
+    const int currentTime = m_d->currentTime();
     KisTimeRange invalidateRange;
 
     if (recursive) {
-        KisTimeRange::calculateTimeRangeRecursive(node, currentTime(), invalidateRange, false);
-    } else if (channel) {
-        const int currentTime = m_d->currentTime();
-        invalidateRange = channel->affectedFrames(currentTime);
+        KisTimeRange::calculateTimeRangeRecursive(node, currentTime, invalidateRange, false);
     } else {
-        invalidateRange = KisTimeRange::infinite(0);
+        invalidateRange = KisTimeRange::calculateNodeAffectedFrames(node, currentTime);
     }
 
-
     // we compress the updated rect (atm, no one uses it anyway)
     QRect unitedRect;
     Q_FOREACH (const QRect &rc, rects) {
