@@ -1055,8 +1055,7 @@ namespace KisLayerUtils {
         while (it != nodes.end()) {
             if ((!allowMasks && !qobject_cast<KisLayer*>(it->data())) ||
                 checkIsChildOf(*it, nodes)) {
-
-                qDebug() << "Skipping node" << ppVar((*it)->name());
+                //qDebug() << "Skipping node" << ppVar((*it)->name());
                 it = nodes.erase(it);
             } else {
                 ++it;
