@@ -45,6 +45,7 @@
 #include <KoXmlWriter.h>
 #include <KoXmlReader.h>
 #include <KoStoreDevice.h>
+#include <KoDialog.h>
 
 #include <klocalizedstring.h>
 #include <kis_debug.h>
@@ -87,7 +88,6 @@
 #include <kis_selection.h>
 #include <kis_fill_painter.h>
 #include <kis_document_undo_store.h>
-#include <kis_painting_assistants_decoration.h>
 #include <kis_idle_watcher.h>
 #include <kis_signal_auto_connection.h>
 #include <kis_debug.h>
@@ -114,6 +114,8 @@
 #include "kis_grid_config.h"
 #include "kis_guides_config.h"
 #include "kis_image_barrier_lock_adapter.h"
+#include "KisReferenceImagesLayer.h"
+
 #include <mutex>
 #include "kis_config_notifier.h"
 #include "kis_async_action_feedback.h"
@@ -234,6 +236,7 @@ public:
         , lastMod(firstMod)
         , nserver(new KisNameServer(1))
         , imageIdleWatcher(2000 /*ms*/)
+        , globalAssistantsColor(KisConfig().defaultAssistantsColor())
         , savingLock(&savingMutex)
         , batchMode(false)
     {
@@ -264,10 +267,12 @@ public:
         , preActivatedNode(0) // the node is from another hierarchy!
         , imageIdleWatcher(2000 /*ms*/)
         , assistants(rhs.assistants) // WARNING: assistants should not store pointers to the document!
+        , globalAssistantsColor(rhs.globalAssistantsColor)
         , gridConfig(rhs.gridConfig)
         , savingLock(&savingMutex)
         , batchMode(rhs.batchMode)
     {
+        // TODO: clone assistants
     }
 
     ~Private() {
@@ -321,6 +326,11 @@ public:
     QScopedPointer<KisSignalAutoConnection> imageIdleConnection;
 
     QList<KisPaintingAssistantSP> assistants;
+
+    QColor globalAssistantsColor;
+
+    KisSharedPtr<KisReferenceImagesLayer> referenceImagesLayer;
+
     KisGridConfig gridConfig;
 
     StdLockableWrapper<QMutex> savingLock;
@@ -431,7 +441,7 @@ KisDocument::KisDocument(const KisDocument &rhs)
 
     // clone the image with keeping the GUIDs of the layers intact
     // NOTE: we expect the image to be locked!
-    setCurrentImage(rhs.image()->clone(true));
+    setCurrentImage(rhs.image()->clone(true), false);
 
     if (rhs.d->preActivatedNode) {
         // since we clone uuid's, we can use them for lacating new
@@ -657,21 +667,25 @@ void KisDocument::setFileBatchMode(const bool batchMode)
 
 KisDocument* KisDocument::lockAndCloneForSaving()
 {
+    // force update of all the asynchronous nodes before cloning
+    QApplication::processEvents();
+    KisLayerUtils::forceAllDelayedNodesUpdate(d->image->root());
+
+    KisMainWindow *window = KisPart::instance()->currentMainwindow();
+    if (window) {
+        if (window->viewManager()) {
+            if (!window->viewManager()->blockUntilOperationsFinished(d->image)) {
+                return 0;
+            }
+        }
+    }
+
     Private::StrippedSafeSavingLocker locker(&d->savingMutex, d->image);
     if (!locker.successfullyLocked()) {
         return 0;
     }
 
-    KisDocument *clonedDoc = new KisDocument(*this);
-
-    // force update of all the asynchronous nodes
-    clonedDoc->image()->waitForDone();
-    QApplication::processEvents();
-
-    KisLayerUtils::forceAllDelayedNodesUpdate(clonedDoc->image()->root());
-    clonedDoc->image()->waitForDone();
-
-    return clonedDoc;
+    return new KisDocument(*this);
 }
 
 bool KisDocument::exportDocumentSync(const QUrl &url, const QByteArray &mimeType, KisPropertiesConfigurationSP exportConfiguration)
@@ -1688,11 +1702,29 @@ QList<KisPaintingAssistantSP> KisDocument::assistants() const
     return d->assistants;
 }
 
-void KisDocument::setAssistants(const QList<KisPaintingAssistantSP> value)
+void KisDocument::setAssistants(const QList<KisPaintingAssistantSP> &value)
 {
     d->assistants = value;
 }
 
+KisSharedPtr<KisReferenceImagesLayer> KisDocument::referenceImagesLayer() const
+{
+    return d->referenceImagesLayer.data();
+}
+
+void KisDocument::setReferenceImagesLayer(KisSharedPtr<KisReferenceImagesLayer> layer, bool updateImage)
+{
+    if (updateImage) {
+        if (layer) {
+            d->image->addNode(layer);
+        } else {
+            d->image->removeNode(d->referenceImagesLayer);
+        }
+    }
+
+    d->referenceImagesLayer = layer;
+}
+
 void KisDocument::setPreActivatedNode(KisNodeSP activatedNode)
 {
     d->preActivatedNode = activatedNode;
@@ -1714,7 +1746,7 @@ KisImageSP KisDocument::savingImage() const
 }
 
 
-void KisDocument::setCurrentImage(KisImageSP image)
+void KisDocument::setCurrentImage(KisImageSP image, bool forceInitialUpdate)
 {
     if (d->image) {
         // Disconnect existing sig/slot connections
@@ -1729,7 +1761,10 @@ void KisDocument::setCurrentImage(KisImageSP image)
     d->shapeController->setImage(image);
     setModified(false);
     connect(d->image, SIGNAL(sigImageModified()), this, SLOT(setImageModified()), Qt::UniqueConnection);
-    d->image->initialRefreshGraph();
+
+    if (forceInitialUpdate) {
+        d->image->initialRefreshGraph();
+    }
 }
 
 void KisDocument::hackPreliminarySetImage(KisImageSP image)
@@ -1760,3 +1795,13 @@ QString KisDocument::exportErrorToUserMessage(KisImportExportFilter::ConversionS
 {
     return errorMessage.isEmpty() ? KisImportExportFilter::conversionStatusString(status) : errorMessage;
 }
+
+void KisDocument::setAssistantsGlobalColor(QColor color)
+{
+    d->globalAssistantsColor = color;
+}
+
+QColor KisDocument::assistantsGlobalColor()
+{
+    return d->globalAssistantsColor;
+}
