@@ -31,10 +31,12 @@
 #include <KoDocumentBase.h>
 #include <kundo2stack.h>
 
+#include <kis_image.h>
 #include <KisImportExportFilter.h>
 #include <kis_properties_configuration.h>
 #include <kis_types.h>
 #include <kis_painting_assistant.h>
+#include <KisReferenceImage.h>
 #include <kis_debug.h>
 #include <KisImportExportUtils.h>
 
@@ -57,13 +59,11 @@ class KoDocumentInfo;
 class KoDocumentInfoDlg;
 class KisImportExportManager;
 class KisUndoStore;
-class KisPaintingAssistant;
 class KisPart;
 class KisGridConfig;
 class KisGuidesConfig;
 class QDomDocument;
-
-class KisPart;
+class KisReferenceImagesLayer;
 
 #define KIS_MIME_TYPE "application/x-krita"
 
@@ -549,7 +549,7 @@ public:
     /**
      * Set the current image to the specified image and turn undo on.
      */
-    void setCurrentImage(KisImageSP image);
+    void setCurrentImage(KisImageSP image, bool forceInitialUpdate = true);
 
     /**
      * Set the image of the document preliminary, before the document
@@ -584,8 +584,24 @@ public:
      */
     KisNodeSP preActivatedNode() const;
 
+    /// @return the list of assistants associated with this document
     QList<KisPaintingAssistantSP> assistants() const;
-    void setAssistants(const QList<KisPaintingAssistantSP> value);
+
+    /// @replace the current list of assistants with @param value
+    void setAssistants(const QList<KisPaintingAssistantSP> &value);
+
+
+    void setAssistantsGlobalColor(QColor color);
+    QColor assistantsGlobalColor();
+
+
+
+    /**
+     * Get existing reference images layer or null if none exists.
+     */
+    KisSharedPtr<KisReferenceImagesLayer> referenceImagesLayer() const;
+
+    void setReferenceImagesLayer(KisSharedPtr<KisReferenceImagesLayer> layer, bool updateImage);
 
     bool save(bool showWarnings, KisPropertiesConfigurationSP exportConfiguration);
 
