@@ -54,6 +54,7 @@
 #include <QStatusBar>
 #include <QMoveEvent>
 #include <QTemporaryFile>
+#include <QMdiSubWindow>
 
 #include <kis_image.h>
 #include <kis_node.h>
@@ -82,6 +83,8 @@
 #include "kis_zoom_manager.h"
 #include "kis_statusbar.h"
 #include "kis_painting_assistants_decoration.h"
+#include "KisReferenceImagesDecoration.h"
+#include "kis_progress_widget.h"
 #include "kis_signal_compressor.h"
 #include "kis_filter_manager.h"
 #include "kis_file_layer.h"
@@ -112,6 +115,7 @@ public:
         , canvas(&viewConverter, resourceManager, _q, document->shapeController())
         , zoomManager(_q, &this->viewConverter, &this->canvasController)
         , paintingAssistantsDecoration(new KisPaintingAssistantsDecoration(_q))
+        , referenceImagesDecoration(new KisReferenceImagesDecoration(_q, document))
         , floatingMessageCompressor(100, KisSignalCompressor::POSTPONE)
     {
     }
@@ -137,10 +141,12 @@ public:
     KisViewManager *viewManager = 0;
     KisNodeSP currentNode;
     KisPaintingAssistantsDecorationSP paintingAssistantsDecoration;
+    KisReferenceImagesDecorationSP referenceImagesDecoration;
     bool isCurrent = false;
     bool showFloatingMessage = false;
     QPointer<KisFloatingMessage> savedFloatingMessage;
     KisSignalCompressor floatingMessageCompressor;
+    QMdiSubWindow *subWindow{nullptr};
 
     bool softProofing = false;
     bool gamutCheck = false;
@@ -244,6 +250,9 @@ KisView::KisView(KisDocument *document, KoCanvasResourceManager *resourceManager
     d->canvas.addDecoration(d->paintingAssistantsDecoration);
     d->paintingAssistantsDecoration->setVisible(true);
 
+    d->canvas.addDecoration(d->referenceImagesDecoration);
+    d->referenceImagesDecoration->setVisible(true);
+
     d->showFloatingMessage = cfg.showCanvasMessages();
 }
 
@@ -305,7 +314,7 @@ void KisView::showFloatingMessageImpl(const QString &message, const QIcon& icon,
 
 bool KisView::canvasIsMirrored() const
 {
-   return d->canvas.xAxisMirrored() || d->canvas.yAxisMirrored();
+    return d->canvas.xAxisMirrored() || d->canvas.yAxisMirrored();
 }
 
 void KisView::setViewManager(KisViewManager *view)
@@ -373,7 +382,7 @@ void KisView::slotContinueAddNode(KisNodeSP newActiveNode)
      */
 
     if (!d->isCurrent &&
-        (!d->currentNode || !d->currentNode->parent())) {
+            (!d->currentNode || !d->currentNode->parent())) {
 
         d->currentNode = newActiveNode;
     }
@@ -512,23 +521,30 @@ void KisView::dropEvent(QDropEvent *event)
             QAction *openInNewDocument = new QAction(i18n("Open in New Document"), &popup);
             QAction *openManyDocuments = new QAction(i18n("Open Many Documents"), &popup);
 
+            QAction *insertAsReferenceImage = new QAction(i18n("Insert as Reference Image"), &popup);
+            QAction *insertAsReferenceImages = new QAction(i18n("Insert as Reference Images"), &popup);
+
             QAction *cancel = new QAction(i18n("Cancel"), &popup);
 
             popup.addAction(insertAsNewLayer);
             popup.addAction(insertAsNewFileLayer);
             popup.addAction(openInNewDocument);
+            popup.addAction(insertAsReferenceImage);
 
             popup.addAction(insertManyLayers);
             popup.addAction(insertManyFileLayers);
             popup.addAction(openManyDocuments);
+            popup.addAction(insertAsReferenceImages);
 
             insertAsNewLayer->setEnabled(image() && urls.count() == 1);
             insertAsNewFileLayer->setEnabled(image() && urls.count() == 1);
             openInNewDocument->setEnabled(urls.count() == 1);
+            insertAsReferenceImage->setEnabled(image() && urls.count() == 1);
 
             insertManyLayers->setEnabled(image() && urls.count() > 1);
             insertManyFileLayers->setEnabled(image() && urls.count() > 1);
             openManyDocuments->setEnabled(urls.count() > 1);
+            insertAsReferenceImages->setEnabled(image() && urls.count() > 1);
 
             popup.addSeparator();
             popup.addAction(cancel);
@@ -560,12 +576,19 @@ void KisView::dropEvent(QDropEvent *event)
                                                                        KisFileLayer::None, image()->nextLayerName(), OPACITY_OPAQUE_U8);
                             adapter.addNode(fileLayer, viewManager()->activeNode()->parent(), viewManager()->activeNode());
                         }
-                        else {
-                            Q_ASSERT(action == openInNewDocument || action == openManyDocuments);
+                        else if (action == openInNewDocument || action == openManyDocuments) {
                             if (mainWindow()) {
                                 mainWindow()->openDocument(url, KisMainWindow::None);
                             }
                         }
+                        else if (action == insertAsReferenceImage || action == insertAsReferenceImages) {
+                            auto *reference = KisReferenceImage::fromFile(url.toLocalFile(), d->viewConverter);
+                            reference->setPosition(d->viewConverter.imageToDocument(cursorPos));
+                            d->referenceImagesDecoration->addReferenceImage(reference);
+
+                            KoToolManager::instance()->switchToolRequested("ToolReferenceImages");
+                        }
+
                     }
                     delete tmp;
                     tmp = 0;
@@ -614,6 +637,11 @@ KisMainWindow * KisView::mainWindow() const
     return dynamic_cast<KisMainWindow *>(window());
 }
 
+void KisView::setSubWindow(QMdiSubWindow *subWindow)
+{
+    d->subWindow = subWindow;
+}
+
 QStatusBar * KisView::statusBar() const
 {
     KisMainWindow *mw = mainWindow();
@@ -623,12 +651,13 @@ QStatusBar * KisView::statusBar() const
 void KisView::slotSavingStatusMessage(const QString &text, int timeout, bool isAutoSaving)
 {
     QStatusBar *sb = statusBar();
-    if (sb)
+    if (sb) {
         sb->showMessage(text, timeout);
+    }
 
     KisConfig cfg;
 
-    if (sb->isHidden() ||
+    if (!sb || sb->isHidden() ||
         (!isAutoSaving && cfg.forceShowSaveMessages()) ||
         (cfg.forceShowAutosaveMessages() && isAutoSaving)) {
 
@@ -639,8 +668,9 @@ void KisView::slotSavingStatusMessage(const QString &text, int timeout, bool isA
 void KisView::slotClearStatusText()
 {
     QStatusBar *sb = statusBar();
-    if (sb)
+    if (sb) {
         sb->clearMessage();
+    }
 }
 
 QList<QAction*> KisView::createChangeUnitActions(bool addPixelUnit)
@@ -651,9 +681,9 @@ QList<QAction*> KisView::createChangeUnitActions(bool addPixelUnit)
 
 void KisView::closeEvent(QCloseEvent *event)
 {
-    // Check whether we're the last view
+    // Check whether we're the last (user visible) view
     int viewCount = KisPart::instance()->viewCount(document());
-    if (viewCount > 1) {
+    if (viewCount > 1 || !isVisible()) {
         // there are others still, so don't bother the user
         event->accept();
         return;
@@ -778,6 +808,34 @@ void KisView::syncLastActiveNodeToDocument()
     }
 }
 
+void KisView::saveViewState(KisPropertiesConfiguration &config) const
+{
+    config.setProperty("file", d->document->url());
+    config.setProperty("window", mainWindow()->windowStateConfig().name());
+
+    if (d->subWindow) {
+        config.setProperty("geometry", d->subWindow->saveGeometry().toBase64());
+    }
+
+    config.setProperty("zoomMode", (int)zoomController()->zoomMode());
+    config.setProperty("zoom", d->canvas.coordinatesConverter()->zoom());
+
+    d->canvasController.saveCanvasState(config);
+}
+
+void KisView::restoreViewState(const KisPropertiesConfiguration &config)
+{
+    if (d->subWindow) {
+        QByteArray geometry = QByteArray::fromBase64(config.getString("geometry", "").toLatin1());
+        d->subWindow->restoreGeometry(QByteArray::fromBase64(geometry));
+    }
+
+    qreal zoom = config.getFloat("zoom", 1.0f);
+    int zoomMode = config.getInt("zoomMode", (int)KoZoomMode::ZOOM_PAGE);
+    d->zoomManager.zoomController()->setZoom((KoZoomMode::Mode)zoomMode, zoom);
+    d->canvasController.restoreCanvasState(config);
+}
+
 void KisView::setCurrentNode(KisNodeSP node)
 {
     d->currentNode = node;
@@ -948,3 +1006,8 @@ void KisView::slotImageSizeChanged(const QPointF &oldStillPoint, const QPointF &
     zoomManager()->updateImageBoundsSnapping();
     zoomManager()->updateGUI();
 }
+
+void KisView::closeView()
+{
+    d->subWindow->close();
+}
