@@ -61,6 +61,9 @@
 #include "KisPart.h"
 #include "kis_shape_layer.h"
 #include <kis_shape_controller.h>
+#include "kis_image_animation_interface.h"
+#include "kis_time_range.h"
+#include "kis_keyframe_channel.h"
 
 
 #include <processing/fill_processing_visitor.h>
@@ -71,7 +74,10 @@
 
 namespace ActionHelper {
 
-    void copyFromDevice(KisViewManager *view, KisPaintDeviceSP device, bool makeSharpClip = false)
+    void copyFromDevice(KisViewManager *view,
+                        KisPaintDeviceSP device,
+                        bool makeSharpClip = false,
+                        const KisTimeRange &range = KisTimeRange())
     {
         KisImageWSP image = view->image();
         if (!image) return;
@@ -126,7 +132,7 @@ namespace ActionHelper {
             }
         }
 
-        KisClipboard::instance()->setClip(clip, rc.topLeft());
+        KisClipboard::instance()->setClip(clip, rc.topLeft(), range);
     }
 
 }
@@ -303,63 +309,69 @@ void KisCutCopyActionFactory::run(bool willCut, bool makeSharpClip, KisViewManag
                 return;
             }
 
-            ActionHelper::copyFromDevice(view, dev, makeSharpClip);
-        }
+            KisTimeRange range;
 
-        if (willCut) {
-            KUndo2Command *command = 0;
-
-            if (node->hasEditablePaintDevice()) {
-                struct ClearSelection : public KisTransactionBasedCommand {
-                    ClearSelection(KisNodeSP node, KisSelectionSP sel)
-                        : m_node(node), m_sel(sel) {}
-                    KisNodeSP m_node;
-                    KisSelectionSP m_sel;
-
-                    KUndo2Command* paint() override {
-                        KisSelectionSP cutSelection = m_sel;
-                        // Shrinking the cutting area was previously used
-                        // for getting seamless cut-paste. Now we use makeSharpClip
-                        // instead.
-                        // QRect originalRect = cutSelection->selectedExactRect();
-                        // static const int preciseSelectionThreshold = 16;
-                        //
-                        // if (originalRect.width() > preciseSelectionThreshold ||
-                        //     originalRect.height() > preciseSelectionThreshold) {
-                        //     cutSelection = new KisSelection(*m_sel);
-                        //     delete cutSelection->flatten();
-                        //
-                        //     KisSelectionFilter* filter = new KisShrinkSelectionFilter(1, 1, false);
-                        //
-                        //     QRect processingRect = filter->changeRect(originalRect);
-                        //     filter->process(cutSelection->pixelSelection(), processingRect);
-                        // }
-
-                        KisTransaction transaction(m_node->paintDevice());
-                        m_node->paintDevice()->clearSelection(cutSelection);
-                        m_node->setDirty(cutSelection->selectedRect());
-                        return transaction.endAndTake();
-                    }
-                };
-
-                command = new ClearSelection(node, selection);
+            KisKeyframeChannel *channel = node->getKeyframeChannel(KisKeyframeChannel::Content.id());
+            if (channel) {
+                const int currentTime = image->animationInterface()->currentTime();
+                range = channel->affectedFrames(currentTime);
             }
 
-            KUndo2MagicString actionName = willCut ?
-                                           kundo2_i18n("Cut") :
-                                           kundo2_i18n("Copy");
-            KisProcessingApplicator *ap = beginAction(view, actionName);
-
-            if (command) {
-                ap->applyCommand(command,
-                                 KisStrokeJobData::SEQUENTIAL,
-                                 KisStrokeJobData::NORMAL);
-            }
-
-            KisOperationConfiguration config(id());
-            config.setProperty("will-cut", willCut);
-            endAction(ap, config.toXML());
+            ActionHelper::copyFromDevice(view, dev, makeSharpClip, range);
         }
+
+        KUndo2Command *command = 0;
+
+        if (willCut && node->hasEditablePaintDevice()) {
+            struct ClearSelection : public KisTransactionBasedCommand {
+                ClearSelection(KisNodeSP node, KisSelectionSP sel)
+                    : m_node(node), m_sel(sel) {}
+                KisNodeSP m_node;
+                KisSelectionSP m_sel;
+
+                KUndo2Command* paint() override {
+                    KisSelectionSP cutSelection = m_sel;
+                    // Shrinking the cutting area was previously used
+                    // for getting seamless cut-paste. Now we use makeSharpClip
+                    // instead.
+                    // QRect originalRect = cutSelection->selectedExactRect();
+                    // static const int preciseSelectionThreshold = 16;
+                    //
+                    // if (originalRect.width() > preciseSelectionThreshold ||
+                    //     originalRect.height() > preciseSelectionThreshold) {
+                    //     cutSelection = new KisSelection(*m_sel);
+                    //     delete cutSelection->flatten();
+                    //
+                    //     KisSelectionFilter* filter = new KisShrinkSelectionFilter(1, 1, false);
+                    //
+                    //     QRect processingRect = filter->changeRect(originalRect);
+                    //     filter->process(cutSelection->pixelSelection(), processingRect);
+                    // }
+
+                    KisTransaction transaction(m_node->paintDevice());
+                    m_node->paintDevice()->clearSelection(cutSelection);
+                    m_node->setDirty(cutSelection->selectedRect());
+                    return transaction.endAndTake();
+                }
+            };
+
+            command = new ClearSelection(node, selection);
+        }
+
+        KUndo2MagicString actionName = willCut ?
+                    kundo2_i18n("Cut") :
+                    kundo2_i18n("Copy");
+        KisProcessingApplicator *ap = beginAction(view, actionName);
+
+        if (command) {
+            ap->applyCommand(command,
+                             KisStrokeJobData::SEQUENTIAL,
+                             KisStrokeJobData::NORMAL);
+        }
+
+        KisOperationConfiguration config(id());
+        config.setProperty("will-cut", willCut);
+        endAction(ap, config.toXML());
     }
 }
 
