@@ -39,6 +39,7 @@
 #include "kis_canvas_decoration.h"
 #include "kis_painting_assistants_decoration.h"
 #include "input/KisInputActionGroup.h"
+#include "KisReferenceImagesDecoration.h"
 
 class KoToolProxy;
 class KoColorProfile;
@@ -163,7 +164,7 @@ public: // KoCanvasBase implementation
     void setInputActionGroupsMask(KisInputActionGroupsMask mask) override;
 
     KisPaintingAssistantsDecorationSP paintingAssistantsDecoration() const;
-
+    KisReferenceImagesDecorationSP referenceImagesDecoration() const;
 
 public: // KisCanvas2 methods
 
@@ -207,8 +208,25 @@ public: // KisCanvas2 methods
     KisAnimationPlayer *animationPlayer() const;
     void refetchDataFromImage();
 
+    /**
+     * @return area of the image (in image coordinates) that is visible on the canvas
+     * with a small margin selected by the user
+     */
+    QRect regionOfInterest() const;
+
+    /**
+     * Set aftificial limit outside which the image will not be rendered
+     * \p rc is measured in image pixels
+     */
+    void setRenderingLimit(const QRect &rc);
+
+    /**
+     * @return aftificial limit outside which the image will not be rendered
+     */
+    QRect renderingLimit() const;
+
 Q_SIGNALS:
-    void imageChanged(KisImageWSP image);
+    void sigCanvasEngineChanged();
 
     void sigCanvasCacheUpdated();
     void sigContinueResizeImage(qint32 w, qint32 h);
@@ -218,6 +236,8 @@ Q_SIGNALS:
     // emitted whenever the canvas widget thinks sketch should update
     void updateCanvasRequested(const QRect &rc);
 
+    void sigRegionOfInterestChanged(const QRect &roi);
+
 public Q_SLOTS:
 
     /// Update the entire canvas area
@@ -234,7 +254,7 @@ public Q_SLOTS:
     void slotSoftProofing(bool softProofing);
     void slotGamutCheck(bool gamutCheck);
     void slotChangeProofingConfig();
-    void slotZoomChanged(int zoom);
+    void slotPopupPaletteRequestedZoomChange(int zoom);
 
     void channelSelectionChanged();
 
@@ -274,6 +294,8 @@ private Q_SLOTS:
 
     void bootstrapFinished();
 
+    void slotUpdateRegionOfInterest();
+
 public:
 
     bool isPopupPaletteVisible() const;
