@@ -36,6 +36,7 @@ class KoStore;
 class KoViewConverter;
 class KoShapeBasedDocumentBase;
 class KoDocumentResourceManager;
+class KisShapeLayerCanvasBase;
 
 const QString KIS_SHAPE_LAYER_ID = "KisShapeLayer";
 /**
@@ -57,7 +58,7 @@ public:
 
     KisShapeLayer(KoShapeBasedDocumentBase* shapeController, KisImageWSP image, const QString &name, quint8 opacity);
     KisShapeLayer(const KisShapeLayer& _rhs);
-    KisShapeLayer(const KisShapeLayer& _rhs, KoShapeBasedDocumentBase* controller);
+    KisShapeLayer(const KisShapeLayer& _rhs, KoShapeBasedDocumentBase* controller, KisShapeLayerCanvasBase *canvas = 0);
     /**
      * Merge constructor.
      *
@@ -67,8 +68,10 @@ public:
      */
     KisShapeLayer(const KisShapeLayer& _merge, const KisShapeLayer &_addShapes);
     ~KisShapeLayer() override;
+protected:
+    KisShapeLayer(KoShapeBasedDocumentBase* shapeController, KisImageWSP image, const QString &name, quint8 opacity, KisShapeLayerCanvasBase *canvas);
 private:
-    void initShapeLayer(KoShapeBasedDocumentBase* controller, KisPaintDeviceSP copyFromProjection = 0);
+    void initShapeLayer(KoShapeBasedDocumentBase* controller, KisPaintDeviceSP copyFromProjection = 0, KisShapeLayerCanvasBase *canvas = 0);
 public:
     KisNodeSP clone() const override {
         return new KisShapeLayer(*this);
@@ -151,6 +154,8 @@ protected:
     friend class ShapeLayerContainerModel;
     KoViewConverter* converter() const;
 
+    KoShapeBasedDocumentBase *shapeController() const;
+
 Q_SIGNALS:
     /**
      * These signals are forwarded from the local shape manager
