@@ -47,12 +47,84 @@
 
 //#define DEBUG_REPAINT
 
+KisShapeLayerCanvasBase::KisShapeLayerCanvasBase(KisShapeLayer *parent, KisImageWSP image)
+    : KoCanvasBase(0)
+    , m_viewConverter(new KisImageViewConverter(image))
+    , m_shapeManager(new KoShapeManager(this))
+    , m_selectedShapesProxy(new KoSelectedShapesProxySimple(m_shapeManager.data()))
+{
+    m_shapeManager->selection()->setActiveLayer(parent);
+}
+
+KoShapeManager *KisShapeLayerCanvasBase::shapeManager() const
+{
+    return m_shapeManager.data();
+}
+
+KoSelectedShapesProxy *KisShapeLayerCanvasBase::selectedShapesProxy() const
+{
+    return m_selectedShapesProxy.data();
+}
+
+KoViewConverter* KisShapeLayerCanvasBase::viewConverter() const
+{
+    return m_viewConverter.data();
+}
+
+void KisShapeLayerCanvasBase::gridSize(QPointF *offset, QSizeF *spacing) const
+{
+    KIS_SAFE_ASSERT_RECOVER_NOOP(false); // This should never be called as this canvas should have no tools.
+    Q_UNUSED(offset);
+    Q_UNUSED(spacing);
+}
+
+bool KisShapeLayerCanvasBase::snapToGrid() const
+{
+    KIS_SAFE_ASSERT_RECOVER_NOOP(false); // This should never be called as this canvas should have no tools.
+    return false;
+}
+
+void KisShapeLayerCanvasBase::addCommand(KUndo2Command *)
+{
+    KIS_SAFE_ASSERT_RECOVER_NOOP(false); // This should never be called as this canvas should have no tools.
+}
+
+
+KoToolProxy * KisShapeLayerCanvasBase::toolProxy() const
+{
+//     KIS_SAFE_ASSERT_RECOVER_NOOP(false); // This should never be called as this canvas should have no tools.
+    return 0;
+}
+
+QWidget* KisShapeLayerCanvasBase::canvasWidget()
+{
+    return 0;
+}
+
+const QWidget* KisShapeLayerCanvasBase::canvasWidget() const
+{
+    return 0;
+}
+
+KoUnit KisShapeLayerCanvasBase::unit() const
+{
+    KIS_SAFE_ASSERT_RECOVER_NOOP(false); // This should never be called as this canvas should have no tools.
+    return KoUnit(KoUnit::Point);
+}
+
+void KisShapeLayerCanvasBase::prepareForDestroying()
+{
+    m_isDestroying = true;
+}
+
+bool KisShapeLayerCanvasBase::hasChangedWhileBeingInvisible()
+{
+    return m_hasChangedWhileBeingInvisible;
+}
+
+
 KisShapeLayerCanvas::KisShapeLayerCanvas(KisShapeLayer *parent, KisImageWSP image)
-        : KoCanvasBase(0)
-        , m_isDestroying(false)
-        , m_viewConverter(new KisImageViewConverter(image))
-        , m_shapeManager(new KoShapeManager(this))
-        , m_selectedShapesProxy(new KoSelectedShapesProxySimple(m_shapeManager.data()))
+        : KisShapeLayerCanvasBase(parent, image)
         , m_projection(0)
         , m_parentLayer(parent)
         , m_asyncUpdateSignalCompressor(100, KisSignalCompressor::FIRST_INACTIVE)
@@ -82,38 +154,6 @@ void KisShapeLayerCanvas::setImage(KisImageWSP image)
     m_viewConverter->setImage(image);
 }
 
-void KisShapeLayerCanvas::prepareForDestroying()
-{
-    m_isDestroying = true;
-}
-
-void KisShapeLayerCanvas::gridSize(QPointF *offset, QSizeF *spacing) const
-{
-    Q_ASSERT(false); // This should never be called as this canvas should have no tools.
-    Q_UNUSED(offset);
-    Q_UNUSED(spacing);
-}
-
-bool KisShapeLayerCanvas::snapToGrid() const
-{
-    Q_ASSERT(false); // This should never be called as this canvas should have no tools.
-    return false;
-}
-
-void KisShapeLayerCanvas::addCommand(KUndo2Command *)
-{
-    Q_ASSERT(false); // This should never be called as this canvas should have no tools.
-}
-
-KoShapeManager *KisShapeLayerCanvas::shapeManager() const
-{
-    return m_shapeManager.data();
-}
-
-KoSelectedShapesProxy *KisShapeLayerCanvas::selectedShapesProxy() const
-{
-    return m_selectedShapesProxy.data();
-}
 
 #ifdef DEBUG_REPAINT
 # include <stdlib.h>
@@ -266,34 +306,6 @@ void KisShapeLayerCanvas::repaint()
     m_hasChangedWhileBeingInvisible |= !m_parentLayer->visible(true);
 }
 
-KoToolProxy * KisShapeLayerCanvas::toolProxy() const
-{
-//     Q_ASSERT(false); // This should never be called as this canvas should have no tools.
-    return 0;
-}
-
-KoViewConverter* KisShapeLayerCanvas::viewConverter() const
-{
-    return m_viewConverter.data();
-}
-
-QWidget* KisShapeLayerCanvas::canvasWidget()
-{
-    return 0;
-}
-
-const QWidget* KisShapeLayerCanvas::canvasWidget() const
-{
-    return 0;
-}
-
-KoUnit KisShapeLayerCanvas::unit() const
-{
-    Q_ASSERT(false); // This should never be called as this canvas should have no tools.
-    return KoUnit(KoUnit::Point);
-}
-
-
 void KisShapeLayerCanvas::forceRepaint()
 {
     /**
@@ -321,11 +333,6 @@ void KisShapeLayerCanvas::resetCache()
     }
 }
 
-bool KisShapeLayerCanvas::hasChangedWhileBeingInvisible()
-{
-    return m_hasChangedWhileBeingInvisible;
-}
-
 void KisShapeLayerCanvas::rerenderAfterBeingInvisible()
 {
     KIS_SAFE_ASSERT_RECOVER_RETURN(m_parentLayer->visible(true))
