@@ -73,6 +73,9 @@ public:
     qreal defImageResolution(bool defaultValue = false) const;
     void defImageResolution(qreal res) const;
 
+    int preferredVectorImportResolutionPPI(bool defaultValue = false) const;
+    void setPreferredVectorImportResolutionPPI(int value) const;
+
     /**
      * @return the id of the default color model used for creating new images.
      */
@@ -248,9 +251,23 @@ public:
     bool showOutlineWhilePainting(bool defaultValue = false) const;
     void setShowOutlineWhilePainting(bool showOutlineWhilePainting) const;
 
+    bool forceAlwaysFullSizedOutline(bool defaultValue = false) const;
+    void setForceAlwaysFullSizedOutline(bool value) const;
+
     bool hideSplashScreen(bool defaultValue = false) const;
     void setHideSplashScreen(bool hideSplashScreen) const;
 
+    enum SessionOnStartup {
+        SOS_BlankSession,
+        SOS_PreviousSession,
+        SOS_ShowSessionManager
+    };
+    SessionOnStartup sessionOnStartup(bool defaultValue = false) const;
+    void setSessionOnStartup(SessionOnStartup value);
+
+    bool saveSessionOnQuit(bool defaultValue) const;
+    void setSaveSessionOnQuit(bool value);
+
     qreal outlineSizeMinimum(bool defaultValue = false) const;
     void setOutlineSizeMinimum(qreal outlineSizeMinimum) const;
 
@@ -544,6 +561,9 @@ public:
     bool calculateAnimationCacheInBackground(bool defaultValue = false) const;
     void setCalculateAnimationCacheInBackground(bool value);
 
+    QColor defaultAssistantsColor(bool defaultValue = false) const;
+    void setDefaultAssistantsColor(const QColor &color) const;
+
     template<class T>
     void writeEntry(const QString& name, const T& value) {
         m_cfg.writeEntry(name, value);
