@@ -51,6 +51,7 @@
 #include <kis_painter.h>
 #include <kis_paint_layer.h>
 #include <KisMimeDatabase.h>
+#include <KisReferenceImagesLayer.h>
 
 #include "KisPart.h"
 #include "canvas/kis_canvas2.h"
@@ -424,9 +425,9 @@ void KisNodeManager::moveNodeAt(KisNodeSP node, KisNodeSP parent, int index)
         if (node->inherits("KisSelectionMask") && parent->inherits("KisLayer")) {
             KisSelectionMask *m = dynamic_cast<KisSelectionMask*>(node.data());
             KisLayer *l = qobject_cast<KisLayer*>(parent.data());
-            KisSelectionMaskSP selMask = l->selectionMask();
-            if (m && m->active() && l && l->selectionMask())
-                selMask->setActive(false);
+            if (m && m->active() && l && l->selectionMask()) {
+                l->selectionMask()->setActive(false);
+            }
         }
         m_d->commandsAdapter.moveNode(node, parent, index);
     }
@@ -621,7 +622,14 @@ void KisNodeManager::slotSomethingActivatedNodeImpl(KisNodeSP node)
 
 void KisNodeManager::slotNonUiActivatedNode(KisNodeSP node)
 {
+    // the node must still be in the graph, some asynchronous
+    // signals may easily break this requirement
+    if (node && !node->graphListener()) {
+        node = 0;
+    }
+
     if (node == activeNode()) return;
+
     slotSomethingActivatedNodeImpl(node);
 
     if (node) {
@@ -634,6 +642,12 @@ void KisNodeManager::slotNonUiActivatedNode(KisNodeSP node)
 
 void KisNodeManager::slotUiActivatedNode(KisNodeSP node)
 {
+    // the node must still be in the graph, some asynchronous
+    // signals may easily break this requirement
+    if (node && !node->graphListener()) {
+        node = 0;
+    }
+
     if (node == activeNode()) return;
 
     slotSomethingActivatedNodeImpl(node);
@@ -765,6 +779,20 @@ KisNodeInsertionAdapter* KisNodeManager::nodeInsertionAdapter() const
     return m_d->nodeInsertionAdapter.data();
 }
 
+bool KisNodeManager::isNodeHidden(KisNodeSP node, bool isGlobalSelectionHidden)
+{
+    if (dynamic_cast<KisReferenceImagesLayer *>(node.data())) {
+        return true;
+    }
+
+    if (isGlobalSelectionHidden && dynamic_cast<KisSelectionMask *>(node.data()) &&
+        (!node->parent() || !node->parent()->parent())) {
+        return true;
+    }
+
+    return false;
+}
+
 void KisNodeManager::nodeOpacityChanged(qreal opacity, bool finalChange)
 {
     KisNodeSP node = activeNode();
@@ -865,10 +893,6 @@ void KisNodeManager::mirrorNodeY()
     mirrorNode(node, commandName, Qt::Vertical);
 }
 
-inline bool checkForGlobalSelection(KisNodeSP node) {
-    return dynamic_cast<KisSelectionMask*>(node.data()) && node->parent() && !node->parent()->parent();
-}
-
 void KisNodeManager::activateNextNode()
 {
     KisNodeSP activeNode = this->activeNode();
@@ -884,7 +908,7 @@ void KisNodeManager::activateNextNode()
         node = activeNode->parent();
     }
 
-    while(node && checkForGlobalSelection(node)) {
+    while(node && isNodeHidden(node, true)) {
         node = node->nextSibling();
     }
 
@@ -912,7 +936,7 @@ void KisNodeManager::activatePreviousNode()
         activeNode = activeNode->parent();
     }
 
-    while(node && checkForGlobalSelection(node)) {
+    while(node && isNodeHidden(node, true)) {
         node = node->prevSibling();
     }
 
@@ -1005,7 +1029,7 @@ void KisNodeManager::Private::saveDeviceAsImage(KisPaintDeviceSP device,
     KoFileDialog dialog(view->mainWindow(), KoFileDialog::SaveFile, "savenodeasimage");
     dialog.setCaption(i18n("Export \"%1\"", defaultName));
     dialog.setDefaultDir(QStandardPaths::writableLocation(QStandardPaths::PicturesLocation));
-    dialog.setMimeTypeFilters(KisImportExportManager::mimeFilter(KisImportExportManager::Export));
+    dialog.setMimeTypeFilters(KisImportExportManager::supportedMimeTypes(KisImportExportManager::Export));
     QString filename = dialog.filename();
 
     if (filename.isEmpty()) return;
