@@ -63,6 +63,9 @@ KisShaderProgram *KisOpenGLShaderLoader::loadShader(QString vertPath, QString fr
 // XXX Check can be removed and set to the MAC version after we move to Qt5.7
 #ifdef Q_OS_OSX
     vertSource.append(KisOpenGL::hasOpenGL3() ? "#version 150 core\n" : "#version 120\n");
+    // OpenColorIO doesn't support the new GLSL version yet.
+    vertSource.append("#define texture2D texture\n");
+    vertSource.append("#define texture3D texture\n");
 #else
     if (KisOpenGL::hasOpenGLES()) {
         vertSource.append("#version 300 es\n");
@@ -85,6 +88,9 @@ KisShaderProgram *KisOpenGLShaderLoader::loadShader(QString vertPath, QString fr
 // XXX Check can be removed and set to the MAC version after we move to Qt5.7
 #ifdef Q_OS_OSX
     fragSource.append(KisOpenGL::hasOpenGL3() ? "#version 150 core\n" : "#version 120\n");
+    // OpenColorIO doesn't support the new GLSL version yet.
+    fragSource.append("#define texture2D texture\n");
+    fragSource.append("#define texture3D texture\n");
 #else
     if (KisOpenGL::hasOpenGLES()) {
         fragSource.append("#version 300 es\nprecision mediump float;\n");
