@@ -23,6 +23,8 @@
 #include <half.h>
 #endif
 
+#include <QByteArray>
+
 #include <kis_debug.h>
 #include <klocalizedstring.h>
 
@@ -101,7 +103,7 @@ public:
     {
 
         //if (m_model="RGBA" || m_colorize) {
-        /*It'd be nice to have LCH automatically selector for LAB in the future, but I don't know how to select LAB 
+        /*It'd be nice to have LCH automatically selector for LAB in the future, but I don't know how to select LAB
          * */
             const RGBPixel* src = reinterpret_cast<const RGBPixel*>(srcU8);
             RGBPixel* dst = reinterpret_cast<RGBPixel*>(dstU8);
@@ -189,7 +191,7 @@ public:
 
                         sat *= (m_adj_s + 1.0);
                         //sat = qBound(0.0, sat, 1.0);
-                        
+
                         intensity += (m_adj_v);
 
                         HCIToRGB(hue/360.0, sat, intensity, &red, &green, &blue);
@@ -221,7 +223,7 @@ public:
                         r = red;
                         g = green;
                         b = blue;
-                        
+
                     } else if (m_type == 4) {
 
                         qreal red = SCALE_TO_FLOAT(src->red);
@@ -265,7 +267,7 @@ public:
             qreal a = SCALE_TO_FLOAT(src->a);
             qreal b = SCALE_TO_FLOAT(src->b);
             qreal L, C, H;
-            
+
             while (nPixels > 0) {
                 if (m_type = 4) {
                     a *= (m_adj_h + 1.0);
@@ -331,7 +333,7 @@ public:
         }
         return -1;
     }
-    
+
     /**
     * name - "h", "s" or "v"
     * (h)ue in range <-1.0, 1.0> ( for user, show as -180, 180 or 0, 360 for colorize)
@@ -383,6 +385,208 @@ private:
     bool m_colorize;
 };
 
+template<typename _channel_type_,typename traits>
+class KisHSVCurveAdjustment : public KoColorTransformation
+{
+    typedef traits RGBTrait;
+    typedef typename RGBTrait::Pixel RGBPixel;
+
+public:
+    KisHSVCurveAdjustment() :
+        m_lumaRed(0.0),
+        m_lumaGreen(0.0),
+        m_lumaBlue(0.0)
+    {}
+
+    QList<QString> parameters() const override
+    {
+      QList<QString> list;
+      list << "curve" << "channel" << "driverChannel" << "relative" << "lumaRed" << "lumaGreen"<< "lumaBlue";
+      return list;
+    }
+
+    int parameterId(const QString& name) const override
+    {
+        if (name == "curve") {
+            return PAR_CURVE;
+        } else if (name == "channel") {
+            return PAR_CHANNEL;
+        } else if (name == "driverChannel") {
+            return PAR_DRIVER_CHANNEL;
+        } else if (name == "relative") {
+            return PAR_RELATIVE;
+        } else if (name == "lumaRed") {
+            return PAR_LUMA_R;
+        } else if (name == "lumaGreen") {
+            return PAR_LUMA_G;
+        } else if (name == "lumaBlue") {
+            return PAR_LUMA_B;
+        }
+        return -1;
+    }
+
+    /**
+    * curve: adjustment curve as QVector<quin16>
+    * channel: which channel to adjust. See KisHSVCurve::ColorChannel.
+    * driverChannel: which channel to use as source for adjustments.
+    * relative:
+    *   false: use curve for direct lookup.
+    *   true: add adjustment to original. In this mode, the curve range is mapped to -1.0 to 1.0
+    * luma Red/Green/Blue: Used for luma calculations.
+    */
+    void setParameter(int id, const QVariant& parameter) override
+    {
+        switch(id)
+        {
+        case PAR_CURVE:
+            m_curve = parameter.value<QVector<quint16>>();
+            break;
+        case PAR_CHANNEL:
+        case PAR_DRIVER_CHANNEL: {
+            int channel = parameter.toInt();
+            KIS_ASSERT_RECOVER_RETURN(0 <= channel && channel < KisHSVCurve::ChannelCount && "Invalid channel. Ignored!");
+
+            if (id == PAR_CHANNEL) {
+                m_channel = channel;
+            } else {
+                m_driverChannel = channel;
+            }
+            } break;
+        case PAR_RELATIVE:
+            m_relative = parameter.toBool();
+            break;
+        case PAR_LUMA_R:
+            m_lumaRed = parameter.toDouble();
+            break;
+        case PAR_LUMA_G:
+            m_lumaGreen = parameter.toDouble();
+            break;
+        case PAR_LUMA_B:
+            m_lumaBlue = parameter.toDouble();
+            break;
+        default:
+            KIS_ASSERT_RECOVER_NOOP(false && "Unknown parameter ID. Ignored!");
+        }
+    }
+
+    void transform(const quint8 *srcU8, quint8 *dstU8, qint32 nPixels) const override
+    {
+        const RGBPixel* src = reinterpret_cast<const RGBPixel*>(srcU8);
+        RGBPixel* dst = reinterpret_cast<RGBPixel*>(dstU8);
+        float max = m_curve.size() - 1;
+
+        int driverChannel = m_relative ? m_driverChannel : m_channel;
+
+        float component[KisHSVCurve::ChannelCount];
+
+        // Aliases for convenience
+        float &h = component[KisHSVCurve::Hue];
+        float &s = component[KisHSVCurve::Saturation];
+        float &v = component[KisHSVCurve::Value];
+        float &r = component[KisHSVCurve::Red];
+        float &g = component[KisHSVCurve::Green];
+        float &b = component[KisHSVCurve::Blue];
+        float &a = component[KisHSVCurve::Alpha];
+
+        while (nPixels > 0) {
+            r = SCALE_TO_FLOAT(src->red);
+            g = SCALE_TO_FLOAT(src->green);
+            b = SCALE_TO_FLOAT(src->blue);
+            a = SCALE_TO_FLOAT(src->alpha);
+
+            RGBToHSV(r, g, b, &h, &s, &v);
+
+            // Normalize hue to 0.0 to 1.0 range
+            h /= 360.0f;
+
+            float adjustment = lookupComponent(component[driverChannel], max);
+
+            if (m_relative) {
+                // Curve uses range 0.0 to 1.0, but for adjustment we need -1.0 to 1.0
+                adjustment = 2.0f * adjustment - 1.0f;
+
+                if (m_channel == KisHSVCurve::AllColors) {
+                    r += adjustment;
+                    g += adjustment;
+                    b += adjustment;
+                } else {
+                    component[m_channel] += adjustment;
+                }
+            } else {
+                if (m_channel == KisHSVCurve::AllColors) {
+                    r = b = g = adjustment;
+                } else {
+                    component[m_channel] = adjustment;
+                }
+            }
+
+            h *= 360.0f;
+            if (h > 360) h -= 360;
+            if (h < 0) h += 360;
+
+            if (m_channel >= KisHSVCurve::Hue) {
+                HSVToRGB(h, s, v, &r, &g, &b);
+            }
+
+            clamp< _channel_type_ >(&r, &g, &b);
+            FLOAT_CLAMP(&a);
+
+            dst->red = SCALE_FROM_FLOAT(r);
+            dst->green = SCALE_FROM_FLOAT(g);
+            dst->blue = SCALE_FROM_FLOAT(b);
+            dst->alpha = SCALE_FROM_FLOAT(a);
+
+            --nPixels;
+            ++src;
+            ++dst;
+        }
+    }
+
+    const float SCALE_FROM_16BIT = 1.0f / 0xFFFF;
+    float lookupComponent(float x, float max) const
+    {
+        // No curve for this component? Pass through unmodified
+        if (max < 2) return x;
+        if (x < 0) return m_curve[0];
+
+        float lookup = x * max;
+        float base = floor(lookup);
+        float offset = lookup - base;
+
+        if (base >= max) {
+            base = max - 1.0f;
+            offset = 1.0f;
+        }
+        int index = (int)base;
+
+        return ((1.0f - offset) * m_curve[index]
+                + offset        * m_curve[index + 1]) * SCALE_FROM_16BIT;
+    }
+
+
+private:
+    enum ParameterID
+    {
+        PAR_CURVE,
+        PAR_CHANNEL,
+        PAR_DRIVER_CHANNEL,
+        PAR_RELATIVE,
+        PAR_LUMA_R,
+        PAR_LUMA_G,
+        PAR_LUMA_B,
+    };
+
+    QVector<quint16> m_curve;
+    int m_channel = 0;
+    int m_driverChannel = 0;
+    bool m_relative = false;
+
+    /* Note: the filter currently only supports HSV, so these are
+     * unused, but will be needed once HSL, etc.
+     */
+    qreal m_lumaRed, m_lumaGreen, m_lumaBlue;
+};
+
 
 KisHSVAdjustmentFactory::KisHSVAdjustmentFactory()
     : KoColorTransformationFactory("hsv_adjustment")
@@ -426,3 +630,47 @@ KoColorTransformation* KisHSVAdjustmentFactory::createTransformation(const KoCol
     return adj;
 
 }
+
+
+KisHSVCurveAdjustmentFactory::KisHSVCurveAdjustmentFactory()
+    : KoColorTransformationFactory("hsv_curve_adjustment")
+{
+}
+
+QList< QPair< KoID, KoID > > KisHSVCurveAdjustmentFactory::supportedModels() const
+{
+    QList< QPair< KoID, KoID > > l;
+    l.append(QPair< KoID, KoID >(RGBAColorModelID , Integer8BitsColorDepthID));
+    l.append(QPair< KoID, KoID >(RGBAColorModelID , Integer16BitsColorDepthID));
+    l.append(QPair< KoID, KoID >(RGBAColorModelID , Float16BitsColorDepthID));
+    l.append(QPair< KoID, KoID >(RGBAColorModelID , Float32BitsColorDepthID));
+    return l;
+}
+
+KoColorTransformation* KisHSVCurveAdjustmentFactory::createTransformation(const KoColorSpace* colorSpace, QHash<QString, QVariant> parameters) const
+{
+    KoColorTransformation * adj;
+    if (colorSpace->colorModelId() != RGBAColorModelID) {
+        dbgKrita << "Unsupported color space " << colorSpace->id() << " in KisHSVCurveAdjustmentFactory::createTransformation";
+        return 0;
+    }
+    if (colorSpace->colorDepthId() == Integer8BitsColorDepthID) {
+        adj = new KisHSVCurveAdjustment< quint8, KoBgrTraits < quint8 > >();
+    } else if (colorSpace->colorDepthId() == Integer16BitsColorDepthID) {
+        adj = new KisHSVCurveAdjustment< quint16, KoBgrTraits < quint16 > >();
+    }
+#ifdef HAVE_OPENEXR
+    else if (colorSpace->colorDepthId() == Float16BitsColorDepthID) {
+        adj = new KisHSVCurveAdjustment< half, KoRgbTraits < half > >();
+    }
+#endif
+    else if (colorSpace->colorDepthId() == Float32BitsColorDepthID) {
+        adj = new KisHSVCurveAdjustment< float, KoRgbTraits < float > >();
+    } else {
+        dbgKrita << "Unsupported color space " << colorSpace->id() << " in KisHSVCurveAdjustmentFactory::createTransformation";
+        return 0;
+    }
+    adj->setParameters(parameters);
+    return adj;
+
+}
