@@ -38,6 +38,8 @@
 #include <kis_paint_layer.h>
 #include <kis_selection.h>
 #include <kis_shape_layer.h>
+#include <KisReferenceImagesLayer.h>
+#include <KisReferenceImage.h>
 #include <kis_file_layer.h>
 #include <kis_clone_layer.h>
 #include <kis_mask.h>
@@ -91,7 +93,14 @@ void KisKraSaveVisitor::setExternalUri(const QString &uri)
 bool KisKraSaveVisitor::visit(KisExternalLayer * layer)
 {
     bool result = false;
-    if (KisShapeLayer* shapeLayer = dynamic_cast<KisShapeLayer*>(layer)) {
+    if (auto* referencesLayer = dynamic_cast<KisReferenceImagesLayer*>(layer)) {
+        Q_FOREACH(KoShape *shape, referencesLayer->shapes()) {
+            auto *reference = dynamic_cast<KisReferenceImage*>(shape);
+            KIS_ASSERT_RECOVER_RETURN_VALUE(reference, false);
+            reference->saveImage(m_store);
+        }
+    }
+    else if (KisShapeLayer *shapeLayer = dynamic_cast<KisShapeLayer*>(layer)) {
         if (!saveMetaData(layer)) {
             m_errorMessages << i18n("Failed to save the metadata for layer %1.", layer->name());
             return false;
