@@ -2,6 +2,7 @@
  *  Copyright (c) 1999 Matthias Elter <elter@kde.org>
  *  Copyright (c) 2002 Patrick Julien <freak@codepimps.org>
  *  Copyright (c) 2010 Lukáš Tvrdý <lukast.dev@gmail.com>
+ *  Copyright (c) 2018 Emmet & Eoin O'Neill <emmetoneill.pdx@gmail.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,16 +22,11 @@
 #ifndef KIS_TOOL_COLOR_PICKER_H_
 #define KIS_TOOL_COLOR_PICKER_H_
 
-#include <QList>
 #include <QTimer>
-
 #include "KoToolFactoryBase.h"
 #include "ui_wdgcolorpicker.h"
 #include "kis_tool.h"
-#include <flake/kis_node_shape.h>
-#include <KoIcon.h>
 #include <kis_icon.h>
-#include <QKeySequence>
 
 class KoResource;
 class KoColorSet;
@@ -51,12 +47,11 @@ public:
 
 class KisToolColorPicker : public KisTool
 {
-
     Q_OBJECT
     Q_PROPERTY(bool toForeground READ toForeground WRITE setToForeground NOTIFY toForegroundChanged)
 
 public:
-    KisToolColorPicker(KoCanvasBase* canvas);
+    KisToolColorPicker(KoCanvasBase *canvas);
     ~KisToolColorPicker() override;
 
 public:
@@ -69,6 +64,7 @@ public:
         bool normaliseValues;
         bool sampleMerged;
         int radius;
+        int blend;
 
         void save(ToolActivation activation) const;
         void load(ToolActivation activation);
@@ -81,7 +77,7 @@ public:
     void continuePrimaryAction(KoPointerEvent *event) override;
     void endPrimaryAction(KoPointerEvent *event) override;
 
-    void paint(QPainter& gc, const KoViewConverter &converter) override;
+    void paint(QPainter &gc, const KoViewConverter &converter) override;
 
     bool toForeground() const;
 
@@ -98,6 +94,7 @@ public Q_SLOTS:
     void slotSetNormaliseValues(bool);
     void slotSetAddPalette(bool);
     void slotChangeRadius(int);
+    void slotChangeBlend(int);
     void slotAddPalette(KoResource* resource);
     void slotSetColorSource(int value);
 
@@ -106,14 +103,15 @@ private:
     void pickColor(const QPointF& pos);
     void updateOptionWidget();
 
-    //Configuration m_config;
+    // Configuration
     QScopedPointer<KisToolUtils::ColorPickerConfig> m_config;
+
     ToolActivation m_toolActivationSource;
     bool m_isActivated;
 
     KoColor m_pickedColor;
 
-    // used to skip some of the tablet events and don't update the colour that often
+    // Used to skip some tablet events and update color less often
     QTimer m_colorPickerDelayTimer;
 
     ColorPickerOptionsWidget *m_optionsWidget;
@@ -123,7 +121,6 @@ private:
 
 class KisToolColorPickerFactory : public KoToolFactoryBase
 {
-
 public:
     KisToolColorPickerFactory()
             : KoToolFactoryBase("KritaSelected/KisToolColorPicker") {
@@ -137,11 +134,9 @@ public:
 
     ~KisToolColorPickerFactory() override {}
 
-    KoToolBase * createTool(KoCanvasBase *canvas) override {
+    KoToolBase *createTool(KoCanvasBase *canvas) override {
         return new KisToolColorPicker(canvas);
     }
 };
 
-
 #endif // KIS_TOOL_COLOR_PICKER_H_
-
